# API ドキュメント

### hs.connect([options], [connectListener])

HandlerSocket サーバに接続します．

 * 引数
   * `options`: (オプション) 接続先を以下のプロパティとして持つオブジェクトです．
     * `host`: 接続先のホスト名または IP アドレス．デフォルトは `'localhost'` です．
     * `port`: 接続先のポート番号です．デフォルトは `9998` です．
     * `auth`: 認証のキーです．

        **HandlerSocket のデフォルト設定では，9998 番ポートは参照のみ可能なことに注意してください．** 9999 番ポートは更新も可能です． 詳細は[HandlerSocketのインストールドキュメント](https://github.com/DeNADev/HandlerSocket-Plugin-for-MySQL/blob/master/docs-ja/installation.ja.txt)を参照してください．

   * `connectListener`:  `'connect'` イベントに対するリスナーとして自動的に加えられます．
 * 戻り値
   * `Connection` オブジェクトを返します．

### hs.in(values...)

IN クライテリアオブジェクトを作成して返します．

 * Parameters
   * `values`: インデックスの値．
 * Returns
   * 新しい IN クライテリアオブジェクト．

### hs.filter(column, op, value)

フィルタオブジェクトを作成して返します．

 * Parameters
   * `column`: このフィルタで使用するカラム．`Connection.openIndex()` で指定された `filterColumns` に含まれていなければなりません．
   * `op`: フィルタの演算子．`'='`, `'>'`, `'>='`, `'<'` and `'<='` のいずれかを指定します．
   * `value`: フィルタの値です．
 * Returns
   * 新しいフィルタオブジェクトを作成して返します．

### hs.while(column, op, value)

 *while* フィルタオブジェクトを作成して返します．

 * Parameters
   * `column`: このフィルタで使用するカラム．
    `Connection.openIndex()` で指定された `filterColumns` に含まれていなければなりません．
   * `op`: フィルタの演算子．`'='`, `'>'`, `'>='`, `'<'` and `'<='` のいずれかを指定します．
   * `value`: フィルタの値です．
 * Returns
   * 新しい *while* フィルタオブジェクトを作成して返します．

## Connection オブジェクト

HandlerSocket への接続を表すオブジェクトで，`EventEmitter` のインスタンスです．

### 'connect' イベント

接続が確立した時に発生するイベントです．

 * コールバック関数: ` function()`

### 'close' イベント

接続が閉じた時に発生するイベントです．

 * コールバック関数: ` function()`

### 'error' イベント

エラーが発生した時に発生するイベントです．

 * コールバック関数: ` function(err)`
   * 引数
     * `err`: エラーを表す `Error` オブジェクトが渡されます．

### Connection.prototype.openIndex(database, table, index, columns, [filterColumns], callback)

接続上で使用するインデックスをオープンします．

 * 引数
   * `database`: データベースの名前です．
   * `table`: テーブルの名前です．
   * `index`: インデックスの名前です．主キーの場合は `'PRIMARY'` を指定します．
   * `columns`: 参照および更新するカラム名の配列です．
   * `filterColumns`: (オプション) フィルタで使用するカラム名の配列です．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, index)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `index`: オープンしたインデックスを表す `Index` オブジェクトが渡されます．

### Connection.prototype.close()

接続を閉じます．

## Index オブジェクト

MySQL 上のインデックスを表すオブジェクトです．

### Index.prototype.find(op, keys, [options], callback)

インデックスを使用して行を取得します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 取得対象となるインデックスの値の配列です．
配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，
同じか少ない数の値を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 取得する最大行数です．省略した場合は `1` となります．
     * `offset`: 取得する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, records)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `records`: 取得した行の配列が渡されます．配列の各要素は `Conection.openIndex()` メソッドの引数 `columns` で指定したカラムの値の配列です．取得した行がなかった場合は空の配列です．複数の行を取得した場合，配列の並びは `op` が `'='`, `'>'`, `'>='` の場合は昇順，`'<'`, `'<='` の場合は降順となります．

### Index.prototype.insert(values, callback)

インデックスを使用して行を挿入します．

 * 引数
   * `values`: 挿入するカラム値の配列です．
配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．

### Index.prototype.update(op, keys, [options], values, callback)

インデックスを使用して行を更新します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
    * `values`: 更新するカラム値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
    * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, numRecords)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `numRecords`: 更新した行数が渡されます．

### Index.prototype.updateAndGet(op, keys, [options], values, callback)

インデックスを使用して行を更新し，それらが更新される前の値を取得します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `values`: 更新するカラム値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, rows)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `records`: 更新される前の行の配列が渡されます．配列の各要素は `Conection.openIndex()` メソッドの引数 `columns` で指定したカラムの値の配列です．更新した行がなかった場合は空の配列です．複数の行を更新した場合，配列の並びは `op` が `'='`, `'>'`, `'>='` の場合は昇順，`'<'`, `'<='` の場合は降順となります．

### Index.prototype.increment(op, keys, [options], values, callback)

インデックスを使用して行をインクリメントします．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
    * `values`: 更新するカラムに加える値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
    * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, numRecords)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `numRecords`: 更新した行数が渡されます．

### Index.prototype.incrementAndGet(op, keys, [options], values, callback)

インデックスを使用して行をインクリメントし，それらが更新される前の値を取得します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `values`: 更新するカラムに加える値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, rows)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `records`: 更新される前の行の配列が渡されます．配列の各要素は `Conection.openIndex()` メソッドの引数 `columns` で指定したカラムの値の配列です．更新した行がなかった場合は空の配列です．複数の行を更新した場合，配列の並びは `op` が `'='`, `'>'`, `'>='` の場合は昇順，`'<'`, `'<='` の場合は降順となります．

### Index.prototype.decrement(op, keys, [options], values, callback)

インデックスを使用して行をデクリメントします．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
    * `values`: 更新するカラムから減じる値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
    * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, numRecords)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `numRecords`: 更新した行数が渡されます．

### Index.prototype.decrementAndGet(op, keys, [options], values, callback)

インデックスを使用して行をデクリメントし，それらが更新される前の値を取得します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 更新対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 更新する最大行数です．省略した場合は `1` となります．
     * `offset`: 更新する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `values`: 更新するカラムから減じる値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `columns` で指定したカラムと同じ数・並び順でなければなりません．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, rows)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `records`: 更新される前の行の配列が渡されます．配列の各要素は `Conection.openIndex()` メソッドの引数 `columns` で指定したカラムの値の配列です．更新した行がなかった場合は空の配列です．複数の行を更新した場合，配列の並びは `op` が `'='`, `'>'`, `'>='` の場合は昇順，`'<'`, `'<='` の場合は降順となります．

### Index.prototype.delete(op, keys, [options], callback)

インデックスを使用して行を削除します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 削除対象となるインデックスの値の配列です．
配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，
同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 削除する最大行数です．省略した場合は `1` となります．
     * `offset`: 削除する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, numRecords)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `numRecords`: 削除した行数が渡されます．

### Index.prototype.deleteAndGet(op, keys, [options], callback)

インデックスを使用して行を削除し，それらが削除される前の値を取得します．

 * 引数
   * `op`: 演算子です．`'='`, `'>'`, `'>='`, `'<'`, `'<='` のいずれかを指定します．
   * `keys`: 削除対象となるインデックスの値の配列です．配列の要素は `Connection.openIndex()` メソッドの引数 `index` で指定したインデックスに定義されたカラム数と同じ並び順で，同じか少ない数を指定することができます．
   * `options`: (オプション) 以下のオプションを含むオブジェクトです．
     * `filters`: `hs.filter()` または `hs.while()` が返すフィルタの配列です．
     * `limit`: 削除する最大行数です．省略した場合は `1` となります．
     * `offset`: 削除する前に読み飛ばす行数です．省略した場合は `0` となります．
   * `callback`: レスポンスを受信した時に呼び出されるコールバック関数です．
 * コールバック関数: `function(err, rows)`
   * 引数
     * `err`: 操作が成功した場合は `null`，失敗した場合は `Error` オブジェクトが渡されます．
     * `records`: 削除される前の行の配列が渡されます．配列の各要素は `Conection.openIndex()` メソッドの引数 `columns` で指定したカラムの値の配列です．削除した行がなかった場合は空の配列です．複数の行を削除した場合，配列の並びは `op` が `'='`, `'>'`, `'>='` の場合は昇順，`'<'`, `'<='` の場合は降順となります．
